//+------------------------------------------------------------------+
//|                                                    FN Signal.mq4 |
//|                                                          Belomor |
//|                                                 belomor@inbox.ru |
//+------------------------------------------------------------------+
#property copyright "Belomor"
#property link      "belomor@inbox.ru"
#property strict
//----
#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1 Green
#property indicator_color2 Red
#property indicator_color3 Silver
//---- input parameters
extern int                FN                  = 34;           // Calculation period
extern ENUM_APPLIED_PRICE Price               = PRICE_CLOSE;  // Price
extern double             Deviation           = 3.0;          // Divations
extern int                FastMAPeriod        =   5;          // Fast average period
extern int                SlowMAPeriod        =  13;          // Slow average period
extern ENUM_MA_METHOD     AverageMethod       = MODE_EMA;     // Averaging method  
extern ENUM_MA_METHOD     NormalizationMethod = MODE_SMA;     // Normalization average method
//---- buffers
double ExtMapBuffer1[];
double ExtMapBuffer2[];
double ExtMapBuffer3[];
double ExtMapBuffer4[];
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- indicators
   IndicatorBuffers(4);
   SetIndexStyle(0,DRAW_HISTOGRAM,STYLE_SOLID,3);
   SetIndexStyle(1,DRAW_HISTOGRAM,STYLE_SOLID,3);
   SetIndexStyle(2,DRAW_LINE,STYLE_DOT);
   SetIndexBuffer(1,ExtMapBuffer2);
   SetIndexBuffer(0,ExtMapBuffer1);
   SetIndexBuffer(2,ExtMapBuffer3);
   SetIndexBuffer(3,ExtMapBuffer4);
   if(FN<2)
      FN=2;
   if(Deviation<0)
      Deviation=1;
   if(FastMAPeriod<1)
      FastMAPeriod=1;
   if(SlowMAPeriod<1)
      SlowMAPeriod=1;
   IndicatorShortName("FNCD ("+(string)FN+","+(string)FastMAPeriod+","+(string)SlowMAPeriod+")");
   SetIndexDrawBegin(0,FN+SlowMAPeriod);
   SetIndexDrawBegin(1,FN+SlowMAPeriod);
   SetIndexLabel(0,"FN Up");
   SetIndexLabel(1,"FN Down");
   SetIndexEmptyValue(0,0);
   SetIndexEmptyValue(1,0);
   IndicatorDigits(4);
//----
   return(0);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double NormalizedX(int F_period, int i)
{
   double result=0;
   if(i<Bars-F_period)
     {
      double C = iMA(NULL,0,1,0,MODE_SMA,Price,i);
      double A = iMA(NULL,0,F_period,0,NormalizationMethod,Price,i);
      double S = iStdDev(NULL,0,F_period,NormalizationMethod,0,Price,i);
         if (S!=0)
               result=(C-A)/S;
     }
   return(result);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double FisherNormalizedX(int F_period, double Dev, int i)
  {
   double result;
   double X;
   if(i<Bars-F_period && Dev>0)
     {
      X=NormalizedX(F_period,i)/Dev;
      if(X>0.99)
         X=0.99;
      if(X<-0.99)
         X=-0.99;
      result=0.5*MathLog((1+X)/(1-X));
     }
   else
      result=0;
   return(result);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   int counted_bars = IndicatorCounted();
      if(counted_bars < 0) return(-1);
      if(counted_bars > 0) counted_bars--;
          int limit=MathMin(Bars-counted_bars,Bars-1);

   //
   //
   //
   //
   //
   
   for(int i=0; i<limit; i++)  ExtMapBuffer4[i]=FisherNormalizedX(FN,Deviation,i);
   for(int i=0; i<limit; i++)  ExtMapBuffer3[i]=iMAOnArray(ExtMapBuffer4,0,SlowMAPeriod,0,AverageMethod,i);
   for(int i=0; i<limit; i++)
   {
      double fastMa = iMAOnArray(ExtMapBuffer4,0,FastMAPeriod,0,AverageMethod,i);
         if(fastMa>ExtMapBuffer3[i])
               { ExtMapBuffer1[i]=fastMa; ExtMapBuffer2[i]=0; }
         else  { ExtMapBuffer2[i]=fastMa; ExtMapBuffer1[i]=0; }
   }
   return(0);
}